#define __BSTD_H__
// I'm willing to break the "never include an include" rule for stdarg.h.
#include <stdarg.h>
// requires: <stdio.h>
// requires: <sys/types.h>  // for time_t definition.

// Housekeeping routines.
//
// Versions of standard library functions which quit on error
// so I don't have to deal with errors most of the time.
//
// Also abstracts some filesystem operations.
//
// A lot of this stuff copied from git-compat-util.h, though some of the
// original contents of this file were only coincidentally similar.
//
// I've preserved __GNUC__ ifdefs, even though my code is egregiously
// nonportable. I use features such as nested functions,
// and variable length arrays.

#ifdef __GNUC__
#define NORETURN __attribute__((__noreturn__))
#else
#define NORETURN
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

#ifndef FLEX_ARRAY
/*
 * See if our compiler is known to support flexible array members.
 */
#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
# define FLEX_ARRAY /* empty */
#elif defined(__GNUC__)
# if (__GNUC__ >= 3)
#  define FLEX_ARRAY /* empty */
# else
#  define FLEX_ARRAY 0 /* older GNU extension */
# endif
#endif

/*
 * Otherwise, default to safer but a bit wasteful traditional style
 */
#ifndef FLEX_ARRAY
# define FLEX_ARRAY 1
#endif
#endif

#ifndef PATH_MAX
#define PATH_MAX 4096
#endif

extern void die_perror(const char *err, ...) NORETURN __attribute__((format (printf, 1, 2)));
extern void die(const char *err, ...) NORETURN __attribute__((format (printf, 1, 2)));
void bmkdir(const char *pathname);
void bchdir(const char *pathname);
void bfclose(FILE *fp);
FILE *bfopen(const char *path, const char *mode);
void bstrtofile(const char *pathname, const char *str);
void bstrappendfile(const char *pathname, const char *str);
void bunlink(const char *pathname);
int direxists(const char *pathname);
int fileexists(const char *pathname);
void bfwrite_bytes(const void *ptr, size_t n, FILE *stream);

// Append list of files in pathname to input list,
// except for ".", "..", and ".gg"
void bopendir(const char *pathname, void (*callback)(const char *));

enum {
  bmode_dir = 0,
  bmode_reg,
  bmode_link,
  bmode_unknown,
};

int blstat(const char *pathname);

enum {
  iobuf_size = 16384,
};

char *bstrdup(const char *s);

// Load a file to memory. Free with unmap call when done.
// Copied from git. Not sure why they don't abstract this function.
// Maybe because of the size parameter?
// Returns NULL if file is empty.
//
// (File is not truly in memory: mmap() loads on demand, but you can't tell!)
void *map_file_to_memory(size_t *len, const char *path);
void unmap_memory(void *map, size_t len);

void bfputc(char c, FILE *fp);
void *mmap_alloc(size_t len);

size_t bfd_size_of(int fd);
void bmake_executable(const char *file);
int bopen(const char *pathname, int flags);
int bopen_write(const char *pathname);
time_t btouch_get_mtime(const char *empty);
char *bfiletostr(int *len, const char *pathname);
extern int prefixcmp(const char *str, const char *prefix);

