#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bstd.h"
#include "id.h"
#include "darray.h"
#include "db.h"
#include "commit.h"

#define NDEBUG
#include <assert.h>

void commit_write(db_ptr db, ID id, commit_t commit) {
  // Compute length of commit object.
  int len = strlen(commit->committer) + 1;
  if (commit->author) len += strlen(commit->author) + 3;
  void addlen(ID p_id) {
    len += 3 + ID_SLEN;
  }
  commit_forall_parent(commit, addlen);
  len += 3 + ID_SLEN; // For the tree.
  len += 3 + ID_SLEN; // For the mode.
  len += commit->messagelen + 1;

  char *s = malloc(len);
  sprintf(s, "%s\n", commit->committer);
  if (commit->author) {
    sprintf(s + strlen(s), "a %s\n", commit->author);
  }
  void printparent(ID p_id) {
    strcat(s, "p ");
    id_sprint(s + strlen(s), p_id);
    strcat(s, "\n");
  }
  commit_forall_parent(commit, printparent);
  strcat(s, "t ");
  id_sprint(s + strlen(s), commit->tree_id);
  strcat(s, "\n");
  if (!id_is0(commit->mode_id)) {
    strcat(s, "x ");
    id_sprint(s + strlen(s), commit->mode_id);
    strcat(s, "\n");
  }
  strcat(s, commit->message);

  assert(strlen(s) <= len);

  id_header_string(id, "c", s);
  db_write_object(db, id, OBJ_COMMIT, (unsigned char *) s, strlen(s));
  free(s);
}

void commit_read(commit_t commit, db_ptr db, ID id) {
  size_t outlen;
  char *outbuf = db_mmap_body_check_type(&outlen, OBJ_COMMIT, db, id);

  commit_clear(commit);
  commit_init(commit);
  id_set0(commit->mode_id);
  // TODO: I ought to limit lengths of some of these fields.
  // TODO: Error check, e.g. truncated commits.

  char *lf = strchr(outbuf, '\n');
  int n = lf - outbuf;  // Don't want the newline.
  commit->committer = malloc(n + 1);
  memcpy(commit->committer, outbuf, n);
  commit->committer[n] = '\0';

  char *cp = lf + 1;
  if (*cp == 'a') {
    cp++;
    if (*cp != ' ') die("invalid commit: expected space after 'a'");
    cp++;
    lf = strchr(cp, '\n');
    n = lf - cp;  // Don't want the newline.
    commit->author = malloc(n + 1);
    memcpy(commit->author, cp, n);
    commit->author[n] = '\0';
    cp = lf + 1;
  }
  while (*cp == 'p') {
    cp++;
    if (*cp != ' ') die("invalid commit: expected space after 'p'");
    cp++;
    ID id;
    id_sread(id, cp);
    commit_add_parent(commit, id);
    cp += ID_SLEN;
    if (*cp != '\n') die("invalid commit: newline after parent expected");
    cp++;
  }
  if (*cp != 't') die("invalid commit: tree not found");
  cp++;
  if (*cp != ' ') die("invalid commit: expected space after 't'");
  cp++;
  id_sread(commit->tree_id, cp);
  cp += ID_SLEN;
  if (*cp != '\n') die("invalid commit: newline after tree expected");
  cp++;

  // Parse permissions and mode.
  if ('x' == *cp) {
    cp++;
    if (*cp != ' ') die("invalid commit: expected space after 'x'");
    cp++;
    id_sread(commit->mode_id, cp);
    cp += ID_SLEN;
    if (*cp != '\n') die("invalid commit: newline after parent expected");
    cp++;
  } else {
    id_set0(commit->mode_id);
  }

  n = outbuf - cp + outlen;
  commit->message = malloc(n + 1);
  commit->messagelen = n;
  memcpy(commit->message, cp, n);
  commit->message[n] = '\0';

  db_munmap(outbuf, outlen);
}

void commit_add_parent(commit_t commit, ID id) {
  darray_append(commit->parent, id_dup(id));
}

void commit_forall_parent(commit_t commit, void (*fn)(ID)) {
  darray_forall(commit->parent, (void (*)(void *)) fn);
}

int commit_parent_count(commit_t commit) {
  return darray_count(commit->parent);
}
