// Commit objects.
//
// To ID a commit, we hash 'c' <body>
//
// Format: <committer> '\n' [author '\n'] ['p' <hash of parent> '\n'...] 
#define __COMMIT_H__
// requires "id.h"
// requires "db.h"
// requires "darray.h"

struct commit_s {
  darray_t parent;
  ID tree_id;
  ID mode_id;
  char *committer;
  char *author;
  char *message;   // ASCIIZ string,
  int messagelen;  // but maybe one day I'll support arbitrary data in commits.
};
typedef struct commit_s commit_t[1];
typedef struct commit_s *commit_ptr;

static inline void commit_init(commit_ptr commit) {
  commit->committer = NULL;
  commit->author = NULL;
  commit->message = NULL;
  darray_init(commit->parent);
}

static inline void commit_clear(commit_ptr commit) {
  free(commit->committer);
  free(commit->author);
  free(commit->message);
  darray_clear_with(commit->parent, free);
}

void commit_read(commit_ptr commit, db_ptr db, ID id);
void commit_write(db_ptr db, ID id, commit_ptr commit);
void commit_add_parent(commit_ptr commit, ID id);
void commit_forall_parent(commit_ptr commit, void (*fn)(ID));
int commit_parent_count(commit_ptr commit);
