#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "bstd.h"
#include "strbuf.h"

void launch_editor(const char *path) {
  const char *editor, *terminal;

  editor = getenv("GIT_EDITOR");
  // if (!editor && editor_program) editor = editor_program;
  if (!editor) editor = getenv("VISUAL");
  if (!editor) editor = getenv("EDITOR");

  terminal = getenv("TERM");
  if (!editor && (!terminal || !strcmp(terminal, "dumb"))) {
    fprintf(stderr,
    "Terminal is dumb but no VISUAL nor EDITOR defined.\n"
    // TODO: -m option
    // "Please supply the message using either -m or -F option.\n"
    );
    exit(1);
  }

  if (!editor) editor = "vi";

  if (strcmp(editor, ":")) {
    size_t len = strlen(editor);
    int i = 0;
    const char *args[6];
    struct strbuf arg0;

    strbuf_init(&arg0, 0);
    if (strcspn(editor, "$ \t'") != len) {
	    /* there are specials */
	    strbuf_addf(&arg0, "%s \"$@\"", editor);
	    args[i++] = "sh";
	    args[i++] = "-c";
	    args[i++] = arg0.buf;
    }
    args[i++] = editor;
    args[i++] = path;
    args[i] = NULL;

    pid_t pid = fork();
    int status;
    if (pid < 0) {
      die_perror("fork");
    } else if (pid) {
      // Parent.
      waitpid(pid, &status, 0);
    } else {
      // Child.
      execvp(args[0], (char * const *) args);
    }
    if (!WIFEXITED(status) || WEXITSTATUS(status)) {
      die("There was a problem with the editor %s.", editor);
    }
    strbuf_release(&arg0);
  }
}
