// Export repository in git-fast-import format

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bstd.h"
#include "id.h"
#include "darray.h"
#include "db.h"
#include "tree.h"
#include "commit.h"

#include "cbt.h"
#include "list.h"

static void export_tree_recurse(const char *pre,
    db_ptr db, ID id, cbt_t xtable) {
  void export_tree(void *p) {
    entry_ptr e = p;
    char *s;
    if (*pre) {
      s = malloc(strlen(pre) + 1 + strlen(e->s) + 2);
      sprintf(s, "%s/%s/", pre, e->s);
    } else {
      s = malloc(strlen(e->s) + 2);
      sprintf(s, "%s/", e->s);
    }
    export_tree_recurse(s, db, e->id, xtable);
    free(s);
  }
  void export_blob(void *p) {
    entry_ptr e = p;
    size_t len;
    int type;
    char *buf = db_mmap_body(&len, &type, db, e->id);
    if (OBJ_BLOB != type) die("bad blob");

    char *s = malloc(strlen(pre) + strlen(e->s) + 1);
    sprintf(s, "%s%s", pre, e->s);
    if (cbt_has(xtable, s)) {
      printf("M 0100755 inline %s\n", s);
    } else {
      printf("M 0100644 inline %s\n", s);
    }
    free(s);
    printf("data %zu\n", len);
    fwrite(buf, 1, len, stdout);
    unmap_memory(buf, len);
    printf("\n");
  }
  tree_t tree;
  tree_init(tree);
  tree_read(tree, db, id);
  darray_forall(tree->tree_list, export_tree);
  darray_forall(tree->blob_list, export_blob);
  tree_clear(tree);
}

static void export_tree(db_ptr db, ID id, cbt_t xtable) {
  export_tree_recurse("", db, id, xtable);
}

void export(db_ptr db, ID start_id) {
  cbt_t mark;
  cbt_init(mark);
  list_t work;
  list_init(work);

  void addwork(ID id) {
    commit_t commit;
    list_append(work, id_dup(id));
    cbt_put_u(mark, (void *) 1, id, 20);
    commit_init(commit);
    commit_read(commit, db, id);
    void addparent(ID id) {
      if (!cbt_has_u(mark, id, 20)) addwork(id);
    }
    commit_forall_parent(commit, addparent);
    commit_clear(commit);
  }

  addwork(start_id);
  while (!list_is_empty(work)) {
    unsigned char *id = list_remove_last(work);
    commit_t commit;
    commit_init(commit);
    commit_read(commit, db, id);

    // TODO: Export other branches, get branch name right.
    puts("commit refs/heads/master");
    if (commit->author) {
      printf("author %s\n", commit->author);
    }
    printf("committer %s\n", commit->committer);
    // TODO: Transmit length instead of EOT kludge.
    puts("data <<EOT");
    printf("%s", commit->message);
    puts("EOT\n\ndeleteall");
    // Read permissions table.
    cbt_t xtable;
    cbt_init(xtable);
    void add(const char *filename) {
      cbt_put(xtable, (void *) 1, filename);
    }
    forall_executables(db, commit->mode_id, add);
    export_tree(db, commit->tree_id, xtable);
    cbt_clear(xtable);
    commit_clear(commit);

    free(id);
  }
  list_clear(work);
  cbt_clear(mark);
}
