// Filesystem tree.

#define __FSTREE_H__
// requires "cbt.h"
// requires "db.h"

struct fstree_s {
  // Map strings to something meaningful to a file.
  cbt_t entry;

  // Map strings to fstree_ptr. Each child represents a subdirectory.
  cbt_t child;
};
typedef struct fstree_s *fstree_ptr;
typedef struct fstree_s fstree_t[1];

static inline void fstree_init(fstree_t f) {
  cbt_init(f->entry);
  cbt_init(f->child);
}

static inline void fstree_clear(fstree_t f) {
  void clear(void *data, const char *key) {
    fstree_clear((fstree_ptr) data);
  }
  cbt_clear(f->entry);
  cbt_forall(f->child, clear);
  cbt_clear(f->child);
}

static inline void fstree_clear_with(fstree_t f, void (*fn)(void *data)) {
  void clear(void *data, const char *key) {
    fstree_clear_with((fstree_ptr) data, fn);
    free(data);
  }
  void fnwrap(void *data, const char *key) {
    fn(data);
  }
  cbt_clear_with(f->entry, fnwrap);
  cbt_clear_with(f->child, clear);
}

void write_tree_from_fstree(db_ptr db, ID id,
    fstree_t f, unsigned char *(*fn)(void *));
