#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bstd.h"
#include "hash.h"
#include "id.h"

void id_cpy(ID dst, const ID src) {
  memcpy(dst, src, 20);
}

unsigned char *id_dup(ID src) {
  unsigned char *new_id = malloc(20);
  memcpy(new_id, src, 20);
  return new_id;
}

void id_fprint(FILE *fp, const ID id) {
  int i;
  for (i = 0; i < 20; i++) {
    fprintf(fp, "%02x", id[i]);
  }
}

void id_sread(ID id, const char *s) {
  int i;
  id_set0(id);
  // Copied from git's setup.c:
  for (i = 0; i < 40; i++) {
    unsigned char c = s[i];
    unsigned char val;
    if (c >= '0' && c <= '9') val = c - '0';
    else if (c >= 'a' && c <= 'f') val = c - 'a' + 10;
    else if (c >= 'A' && c <='F') {
      val = c - 'A' + 10;
      c -= 'A' - 'a';
    } else {
      die("bad id: '%s'", s);
      val = 0;
    }
    if (!(i & 1)) val <<= 4;
    id[i >> 1] |= val;
  }
}

void id_fread(ID id, FILE *fp) {
  char s[40];
  int n = fread(s, 1, 40, fp);
  if (n != 40) die_perror("fread");
  id_sread(id, s);
}

void id_sprint(char *s, const ID id) {
  int i;
  for (i = 0; i < 20; i++) {
    sprintf(s + 2 * i, "%02x", id[i]);
  }
}

void id_set0(ID id) {
  memset(id, 0, 20);
}

int id_is0(const ID id) {
  int i;
  for (i = 0; i < 20; i++) {
    if (id[i]) return 0;
  }
  return 1;
}

int id_cmp(const ID id1, const ID id2) {
  return memcmp(id1, id2, 20);
}

// Return 0 if ID read successfully.
// Return 1 if EOF encountered at start.
// Die with error otherwise.
int id_fread_eof(ID id, FILE *fp) {
  char s[40];
  int n = fread(s, 1, 40, fp);
  if (n != 40) {
    if (!n && feof(fp)) return 1;
    die_perror("fread");
  }
  id_sread(id, s);
  return 0;
}

void id_header_string(ID id, const char *hdr, const char *s) {
  SHA1_CTX context;
  SHA1Init(&context);
  SHA1Update(&context, (unsigned char *) hdr, strlen(hdr));
  SHA1Update(&context, (unsigned char *) s, strlen(s));
  SHA1Final(id, &context);
}

void id_header_buflen(ID id, const char *hdr,
    const unsigned char *buf, size_t len) {
  SHA1_CTX context;
  SHA1Init(&context);
  SHA1Update(&context, (unsigned char *) hdr, strlen(hdr));
  SHA1Update(&context, (unsigned char *) buf, len);
  SHA1Final(id, &context);
}

void id_blob_buflen(ID id, const unsigned char *s, int len) {
  SHA1_CTX context;
  SHA1Init(&context);
  SHA1Update(&context, (unsigned char *) "b", 1);
  SHA1Update(&context, (unsigned char *) s, len);
  SHA1Final(id, &context);
}


void id_string(ID id, const char *s) {
  SHA1_CTX context;
  SHA1Init(&context);
  SHA1Update(&context, (unsigned char *) s, strlen(s));
  SHA1Final(id, &context);
}

void id_buflen(ID id, const unsigned char *s, int len) {
  SHA1_CTX context;
  SHA1Init(&context);
  SHA1Update(&context, (unsigned char *) s, len);
  SHA1Final(id, &context);
}
