#define __INDEX_H__
// requires <stdint.h>
// requires "db.h"
// requires "cbt.h"
// requires "id.h"
// requires "fstree.h"

// One reason git is so fast is that it doesn't bother looking at files to see
// if their content has changed. Instead, results of stat calls are cached and
// reexamined.
//
// The "index" is a cache holding results of old stat calls and also hashes of
// files. Provided we can trust the system clock, we can avoid opening files
// so long as their times of last modification are unchanged.
//
// We also examine a few other stats as backup measures.
// Unlike git, we do not use ownership or dev as heuristics.

struct index_entry_s {
  uint32_t ctime;
  uint32_t mtime;
  uint32_t mode;
  uint32_t inode;
  uint32_t size;
  int dodgy;
  ID id;
};
typedef struct index_entry_s *index_entry_ptr;
typedef struct index_entry_s index_entry_t[1];

struct index_s {
  fstree_t root;
  int has_read_timestamp;
  time_t timestamp;
  char *file;
};
typedef struct index_s index_t[1];
// Already defined in db.h:
// typedef struct index_s *index_ptr;

void index_init(index_t index, const char *index_file);
void index_clear(index_t index);
void index_add_path(index_t index, db_ptr db, const char *fullname);
void index_write(const index_ptr index);
void index_read(index_t index);
int index_entry_cmp(index_t index, index_entry_ptr ie, const char *pathname);
int fstree_update_path(index_t index, db_ptr db, fstree_ptr f,
    index_entry_ptr ie, const char *pathname);
index_entry_ptr index_entry_dup(const index_entry_ptr ie);
