#include <assert.h>
#include <stdlib.h>
#include "list.h"

void list_init(list_ptr l)
{
    l->first = NULL;
    l->last = NULL;
    l->count = 0;
}

void list_clear(list_ptr l) {
  list_node_ptr node = l->first;
  while (node) {
    list_node_ptr next = node->next;
    free(node);
    node = next;
  }
  l->first = NULL;
  l->last = NULL;
  l->count = 0;
}

void list_clear_with(list_ptr l, void (*fn)(void *)) {
  list_node_ptr node = l->first;
  while (node) {
    list_node_ptr next = node->next;
    fn(node->data);
    free(node);
    node = next;
  }
  l->first = NULL;
  l->last = NULL;
  l->count = 0;
}

list_node_ptr new_list_node(void *data) {
  list_node_ptr node = malloc(sizeof(*node));
  node->data = data;
  return node;
}

void list_insert(list_ptr list, void *data)
{
    list->count++;
    list_node_ptr node = new_list_node(data);
    node->next = list->first;
    node->prev = NULL;
    list->first = node;
    if (node->next) node->next->prev = node;
    else list->last = node;
}

void list_append(list_ptr list, void *data) {
    list->count++;
    list_node_ptr node = new_list_node(data);
    node->prev = list->last;
    node->next = NULL;
    list->last = node;
    if (node->prev) node->prev->next = node;
    else list->first = node;
}

void list_forall(list_ptr l, void (*f)(void *)) {
    list_node_ptr p;
    for(p = l->first; p; p = p->next) f(p->data);
}

void *list_remove_last(list_ptr l) {
    if (l->last) {
        l->count--;
        list_node_ptr p = l->last;
	l->last = p->prev;
	if (p->prev) p->prev->next = NULL;
	else l->first = NULL;
	void *data = p->data;
	free(p);
	return data;
    }
    return NULL;
}
