// Tree objects. Differences with git:
//  - We order them by object type. Trees first, blobs second.
//    Then lexicographically.
//  - No permissions field.
//
// To ID a tree, we hash 't' <body>.
//
// Body format: <list of trees> '\n' <list of blobs>
//
// The lists have the format:
//   SHA1 ' ' string '\n'
// Each list is sorted by the string field.

#define __TREE_H__
// requires "darray.h"
// requires "id.h"
// requires "db.h"

struct entry_s {
  ID id;
  char *s;
};
typedef struct entry_s *entry_ptr;
typedef struct entry_s entry_t[1];

struct tree_s {
  darray_t tree_list, blob_list;
};
typedef struct tree_s *tree_ptr;
typedef struct tree_s tree_t[1];

void entry_init(entry_t e, const ID id, const char *s);
void entry_clear(entry_t e);
void tree_init(tree_t tree);
void tree_clear(tree_t tree);

// Both of these assume 's' does not appear on either list yet.
void tree_append_tree(tree_t tree, const ID id, const char *s);
void tree_append_blob(tree_t tree, const ID id, const char *s);

// Read body of tree. Header should already have been read.
void tree_read(tree_t tree, db_ptr db, const ID id);
void compute_id_and_write_tree(db_ptr db, ID id, tree_t tree);
